'=========================================================================
' Tastaturgestuetztes Textbasiertes User Interface (TUI)fuer die Konsole 
' oder Graphik (8x16 Zeichensatz)
' Erstellt 14.07.06
' Letzte Ueberarbeitung am 04.12.11
'=========================================================================


#INCLUDE ONCE "Common.bi"


DECLARE SUB Zentriert(Zeile AS INTEGER, Text AS STRING)
DECLARE SUB ClScreen(o AS INTEGER, l AS INTEGER, u AS INTEGER, r AS INTEGER)
DECLARE SUB Rahmen(Titel AS STRING, Fusszeile AS STRING, o AS INTEGER, l AS INTEGER, _
                   u AS INTEGER, r AS INTEGER)
DECLARE SUB SetCursor                   
DECLARE SUB InString(BYREF s AS STRING, Laenge AS INTEGER, BYREF ExitCode AS INTEGER)
DECLARE SUB Edit OVERLOAD (BYREF s AS STRING, Laenge AS INTEGER = 255)
DECLARE SUB Edit(BYREF a AS SINGLE)
DECLARE SUB Edit(BYREF k AS INTEGER)
DECLARE SUB EditList (s() AS STRING, Laenge AS INTEGER)
DECLARE SUB EditTab(s() AS STRING, SpBreite() AS INTEGER)
DECLARE SUB Eingabe(Maske() AS STRING, s() AS STRING, Laenge AS INTEGER)
DECLARE SUB Menue(s() AS STRING, BYREF sNr AS INTEGER)


SUB Zentriert(Zeile AS INTEGER, Text AS STRING)
' Zentrierter Text

  DIM AS INTEGER Breite

  Breite = LOWORD(WIDTH)
  LOCATE Zeile, (Breite - LEN(Text)) / 2
  PRINT Text

END SUB


SUB ClScreen(o AS INTEGER, l AS INTEGER, u AS INTEGER, r AS INTEGER)
' Loescht einen Bildschirmausschnitt durch Ueberschreiben mit der
' Hintergrundfarbe.
' - o  = Position oberer Rand
' - l  = Position linker Rand
' - u  = Position unterer Rand
' - r  = Position rechter Rand

  DIM AS INTEGER Breite, y

  Breite = r - l + 1
  FOR y = o TO u
    LOCATE y, l: PRINT SPACE(Breite);
  NEXT y

END SUB


SUB Rahmen(Titel AS STRING, Fusszeile AS STRING, o AS INTEGER, l AS INTEGER, _
  u AS INTEGER, r AS INTEGER)
' Zeichnet einen Rahmen mit Titel.
' - Titel = Titel, zentriert (falls Titel <> "")
' - Fusszeile = Fusszeile, zentriert (falls Fusszeile <> "")
' - o  = Position oberer Rand
' - l  = Position linker Rand
' - u  = Position unterer Rand
' - r  = Position rechter Rand
' Der Rahmen wird zum Loeschen des Hintergrundes mit aktuellen Hintergrundfarbe
' gefuellt. Erfolgt vorher eine entsprechende COLOR-Anweisung, wird ein farbiges
' Fenster mit Rahmen gezeichnet.

  DIM AS INTEGER Breite, Hoehe, y
  DIM AS STRING*1 eol, eor, eul, eur, hlin, vlin

  eol = CHR(218)
  eor = CHR(191)
  eul = CHR(192)
  eur = CHR(217)
  hlin = CHR(196)
  vlin = CHR(179)

  ' Hintergrund loeschen:
  ClScreen(o, l, u, r)

  ' Rahmen zeichnen:
  LOCATE o, l: PRINT eol; STRING(r - l - 1, hlin); eor;
  FOR y = o + 1 TO u - 1
    LOCATE y, l: PRINT vlin;
    LOCATE y, r: PRINT vlin;
  NEXT y
  LOCATE u, l: PRINT eul; STRING(r - l - 1, hlin); eur;

  ' Titel und Fusszeile einfuegen:
  Breite = r - l + 1
  IF Titel <> "" THEN
    LOCATE o, l + (Breite - LEN(Titel) - 2) / 2
    PRINT SPACE(1); Titel; SPACE(1);
  END IF
  IF Fusszeile <> "" THEN
    LOCATE u, l + (Breite - LEN(Fusszeile) - 2) / 2
    PRINT SPACE(1); Fusszeile; SPACE(1);
  END IF

END SUB


PRIVATE SUB SetCursor
' Markiert im Graphik-Mode die Cursorposition durch einen Unterstrich.
  
  DIM AS INTEGER row, col, x, y
  
  row = CSRLIN
  col = POS
  x = 8 * (col - 1)
  y = 16 * (row - 1)
  DRAW STRING (x, y), "_"
    
END SUB


SUB InString(BYREF s AS STRING, Laenge AS INTEGER, BYREF ExitCode AS INTEGER)
' Prozedur zum Editieren eines String mit Vorbelegung und Laengenbegrenzung.
' Parameter:
' - s = zu editierender String
' - Laenge = Maximale Laenge von s. Scrollen wird nicht unterstuetzt, d.h. die
'   moegliche Laenge von s ist durch das Ende der unteren Fensterzeile begrenzt.
' - ExitCode = Tastencode, mit dem die SUB verlassen wurde (SC_ENTER, SC_ESC,
'   SC_UP, SC_DOWN, SC_TAB oder SC_SHIFT_TAB).

  DIM AS INTEGER Zeile, Spalte, Key, CPos, i, Crow, Ccol, MaxLen
  DIM AS STRING driver
  
  'Falls Graphikbildschirm, urspruengliche Parameter wieder herstellen, sonst
  'funktioniert des Setzen des Cursors nicht:
  SCREENINFO ,,,,,,driver
  IF driver <> "" THEN
    VIEW
    WINDOW
  END IF
  
  Zeile = CSRLIN
  Spalte = POS
  MaxLen = LOWORD(WIDTH) - Spalte + LOWORD(WIDTH) * (HIWORD(VIEW PRINT()) - Zeile)
  
  CPos = LEN(s)
      
  'Editieren:
  DO
    
    's anzeigen und den Cursor setzen:
    LOCATE Zeile, Spalte
    PRINT SPACE(LEN(s) + 1);
    LOCATE Zeile, Spalte
    FOR i = 0 TO LEN(s)
      PRINT MID(s, i, 1);      
      IF i = CPos THEN
        Crow = CSRLIN
        Ccol = POS
      END IF      
    NEXT i
    LOCATE Crow, Ccol 
    IF driver <> "" THEN SetCursor
    
    'Tasten auswerten:
    Key = GETKEY
    SELECT CASE Key
    CASE SC_ENTER, SC_ESC, SC_UP, SC_DOWN, SC_TAB, SC_SHIFT_TAB
      'Ende
      ExitCode = key
      EXIT DO 
    CASE SC_LEFT  
      'Eine Position nach links
      IF CPos > 0 THEN CPos = CPos - 1
    CASE SC_RIGHT
      'Eine Position nach rechts
      IF CPos < LEN(s) THEN CPos = CPos + 1
    CASE SC_END
      'Zum Ende von s
      CPos = LEN(s)
    CASE SC_HOME
      'Zum Anfang von s
      CPos = 0
    CASE SC_DEL
      IF LEN(s) > 0 AND CPos < LEN(s) THEN _
        s = LEFT(s, CPos) + RIGHT(s, LEN(s) - CPos - 1)
    CASE SC_BKSP
      IF LEN(s) > 0 AND CPos > 0 THEN
        s = LEFT(s, CPos - 1) + RIGHT(s, (LEN(s) - CPos))
        LOCATE Zeile, Spalte
        PRINT s + SPACE(2);  'putzt auch den Cursor mit weg, falls er hinten steht
        CPos = CPos - 1
      END IF
    CASE 32 TO 255
      'druckbare Zeichen
      IF LEN(s) < Laenge AND LEN(s) < MaxLen THEN
        s = LEFT(s, CPos) + CHR(Key) + RIGHT(s, LEN(s) - CPos)
        CPos = CPos + 1
      END IF
    END SELECT
    
  LOOP
  
  'Ende:
  LOCATE Zeile, Spalte
  PRINT s + SPACE(1);  'putzt auch den Cursor mit weg, falls er hinten steht
  
END SUB


SUB Edit(BYREF s AS STRING, Laenge AS INTEGER = 255)
' Prozedur zum Editieren eines Strings (Ersatz fuer die INPUT-Anweisung). 
' Beenden mit ENTER. Parameter Laenge (optional) siehe SUB InString.

  DIM AS INTEGER Zeile, Spalte, ExitCode

  Zeile = CSRLIN
  Spalte = POS
  DO
    LOCATE Zeile, Spalte
    InString(s, Laenge, ExitCode)
  LOOP UNTIL ExitCode = SC_ENTER
  PRINT 'neue Zeile

END SUB


SUB Edit(BYREF a AS SINGLE)
' Eingabe einer Gleitkommazahl mit Vorbelegung. Eventuelles Dezimalkomma wird 
' in Dezimalpunkt umgewandelt.

  DIM AS STRING aStr
  DIM AS INTEGER Zeile, Spalte, ExitCode, k

  'a in String umwandeln und editieren:
  Zeile = CSRLIN
  Spalte = POS
  IF a >= 0 THEN aStr = SPACE(1) + STR(a) ELSE aStr = STR(a)
  DO
    LOCATE Zeile, Spalte
    InString(aStr, 15, ExitCode)
  LOOP UNTIL ExitCode = SC_ENTER
  
  'Editierten String in Zahl umwandeln und zur Kontrolle anzeigen:
  k = INSTR(aStr, ",")
  IF k > 0 THEN MID(aStr, k) = "."
  a = VAL(aStr)
  LOCATE Zeile, Spalte: PRINT SPACE(LEN(aStr));
  LOCATE Zeile, Spalte: PRINT a 'erst jetzt kommt der Zeilenvorschub!  

END SUB


SUB Edit(BYREF k AS INTEGER)
  ' Editieren einer Ganzzahl mit Vorbelegung

  DIM AS STRING kStr
  DIM AS INTEGER Zeile, Spalte, ExitCode

  'a in String umwandeln und editieren:
  Zeile = CSRLIN
  Spalte = POS
  IF k >= 0 THEN kStr = SPACE(1) + STR(k) ELSE kStr = STR(k)
  DO
    LOCATE Zeile, Spalte
    InString(kStr, 11, ExitCode)
  LOOP UNTIL ExitCode = SC_ENTER
  
  'Editierten String in Zahl umwandeln und zur Kontrolle anzeigen:
  k = VAL(kStr)
  LOCATE Zeile, Spalte: PRINT SPACE(LEN(kStr));
  LOCATE Zeile, Spalte: PRINT k 'erst jetzt kommt der Zeilenvorschub!  

END SUB


SUB EditList (s() AS STRING, Laenge AS INTEGER)
' Editieren einer Variablenliste s(0) bis s(n). Kann auch als "Editor" verwendet
' werden. Die obere linke Ecke des Eingabefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden (gilt auch fuer die nachfolgenden Prozeduren).
' Parameter:
' - s(0) bis s(n) = Textzeilen
' - Laenge siehe SUB InString
' Verlassen der Prozedur erfolgt mit Esc.

  DIM AS INTEGER Zeile, Spalte, i, n, ExitCode

  Zeile = CSRLIN
  Spalte = POS
  n = UBOUND(s)

  FOR i = 0 TO n
    LOCATE Zeile + i, Spalte: PRINT s(i);
  NEXT i

  i = 0
  DO
    LOCATE Zeile + i, Spalte
    InString(s(i), Laenge, ExitCode)
    SELECT CASE ExitCode
    CASE SC_DOWN, SC_ENTER, SC_TAB
      'Eine Zeile weiter:
      IF i < n THEN i = i + 1 ELSE i = 0
    CASE SC_UP, SC_SHIFT_TAB
      'Eine Zeile zurueck:
      IF i > 0 THEN i = i - 1 ELSE i = n
    CASE SC_ESC
      'Ende:
      EXIT DO
    END SELECT
  LOOP

END SUB


SUB EditTab(s() AS STRING, SpBreite() AS INTEGER)
' Tabelle editieren. Parameter:
' - s(0, 0) bis s(m, n) = Variablen (Zeile 0 bis m, Spalte 0 bis n)
' - SpBreite(0) bis SpBreite(n) = Spaltenbreiten (entspricht dem Parameter Laenge
'   fuer jede Variable, siehe SUB InString)
' Die linke obere Ecke des Eingabefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden.
' Verlassen der Prozedur erfolgt mit Esc.

  DIM AS INTEGER Zeile, Spalte, m, n, i, j, ExitCode

  Zeile = CSRLIN
  Spalte = POS
  m = UBOUND(s, 1)
  n = UBOUND(s, 2)
  DIM AS INTEGER col(n)
  
  'Berechnung der Spaltenpositionen:
  col(0) = Spalte
  FOR j = 1 TO n
    col(j) = col(j - 1) + SpBreite(j - 1)
  NEXT j
  
  FOR i = 0 TO m
    FOR j = 0 TO n
      LOCATE Zeile + i, col(j)
      PRINT s(i, j)
    NEXT j
  NEXT i

  i = 0
  j = 0
  DO
    LOCATE Zeile + i, col(j): InString(s(i, j), SpBreite(j) - 1, ExitCode)
    SELECT CASE ExitCode
    CASE SC_ENTER
      'Zeile fuer Zeile von einer Spalte zur naechsten:
      IF j < n THEN 
        j = j + 1 
      ELSEIF i < m THEN
        i = i + 1
        j = 0
      ELSE
        i = 0
        j = 0
      END IF
    CASE SC_TAB: IF j < n THEN j = j + 1 ELSE j = 0
    CASE SC_SHIFT_TAB: IF j > 0 THEN j = j - 1 ELSE j = n
    CASE SC_UP: IF i > 0 THEN i = i - 1 ELSE i = m
    CASE SC_DOWN: IF i < m THEN i = i + 1 ELSE i = 0
    CASE SC_ESC: EXIT DO
    END SELECT
  LOOP

END SUB


SUB Eingabe(Maske() AS STRING, s() AS STRING, Laenge AS INTEGER)
' Eingabemaske. Parameter:
' - Maske(0) bis Maske(n) = Eingabemaske (muessen alle gleich lang sein, z.B.
'   mit Leerzeichen auffuellen).
' - s(0) bis s(n) = zu editierende Variablen
' - Laenge(0) bis Laenge(n) = maximale Eingabelaenge fuer jede Variable.
' Die linke obere Ecke des Eingabefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden.
' Verlassen erfolgt mit ESC.

  DIM AS INTEGER Zeile, Spalte, i, n, col

  Zeile = CSRLIN
  Spalte = POS
  n = UBOUND(s)

  'Maske:
  FOR i = 0 TO n
    LOCATE Zeile + i, Spalte: PRINT Maske(i);
    col = POS
  NEXT i

  'Variablen anzeigen und editieren:
  
  LOCATE Zeile, col
  EditList(s(), Laenge)

END SUB


SUB Menue(s() AS STRING, BYREF sNr AS INTEGER)
' Menue. Parameter:
' - s(0) bis s(n) = Menue-Eintraege (sollten alle gleich lang sein, z.B. mit
'   Leerzeichen auffuellen)
' - sNr = Nummer des gewaehlten Menue-Eintrages
' Die linke obere Ecke des Menuefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden.

  DIM AS INTEGER Zeile, Spalte, fgcolor, bkcolor, i, j, n, key

  fgcolor = LOWORD(COLOR)
  bkcolor = HIWORD(COLOR) 
  Zeile = CSRLIN
  Spalte = POS
  n = UBOUND(s)
  
  i = 0
  DO
    
    FOR j = 0 TO n
      IF j = i THEN 
        IF bkcolor = 7 OR bkcolor = 15 THEN COLOR 15, 3 ELSE COLOR 0, 7
      ELSE
        COLOR fgcolor, bkcolor
      END IF
      LOCATE Zeile + j, Spalte
      PRINT s(j);
    NEXT j
    
    key = GETKEY
    SELECT CASE key
      CASE SC_DOWN
        IF i < n THEN i = i + 1 ELSE i = 0
      CASE SC_UP
        IF i > 0 THEN i = i - 1 ELSE i = n
      CASE SC_ENTER
        EXIT DO          
    END SELECT

  LOOP
  
  SNr = i
  LOCATE ,,1  'Cursor wieder ein
  
END SUB
